/*-
 * Copyright (c) 2020-2021 Carsten Sonne Larsen <cs@innolan.net>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

#ifndef TEXT_H_INCLUDED
#define TEXT_H_INCLUDED

#include "config.h"

// Settings
const char *prefsDir;
const char *prefsFile;
const char *persistentPrefsDir;
const char *persistentPrefsFile;
const char *prefsFileSearch;
const char *prefsFileFound;
const char *prefsFileNotFound;
const char *prefsFileSave;
const char *fileOpenError;
const char *fileSaveError;
const char *fileReadError;
const char *fileWriteError;
const char *unknownSetting;
const char *foundSetting;
const char *foundWbSetting;
const char *foundCliSetting;
const char *integerError;
const char *yesNoError;
const char *applyDefaultSettings;
const char *applyFileSettings;
const char *applyCliSettings;
const char *applyWbSettings;
const char *effectiveSettings;
const char *settingChangedLong;
const char *settingChangedString;
const char *settingSetLong;
const char *settingSetString;
const char *settingTooLow;
const char *settingTooHigh;
const char *settingGreaterThan;
const char *settingInvalid;
const char *settingValueLong;
const char *settingValueString;
const char *saveValueLong;
const char *saveValueString;
const char *noValueString;
const char *yesValueString;
const char *falseValueString;
const char *trueValueString;
const char *validatingSettings;
const char *validatedSettings;

// Library

// Time zone
const char *applyTimeZone;
const char *tzSettingValid;
const char *tzSettingInvalid;
const char *tzFromEnv;
const char *tzFromSet;
const char *tzWorkbench;
const char *tzPreferences;
const char *tzChange;
const char *tzDstAbbr;
const char *tzStdAbbr;
const char *timezoneCur;
const char *timezoneStd;
const char *timezoneDst;
const char *timezoneNoTrans;
const char *tzChecker;
const char *tzCheck;
const char *tzTrans;

// Concat text
const char *Text2P;

// Common text
const char *TextEmpty;
const char *TextStarting;
const char *TextRestarting;
const char *TextStarted;
const char *TextStopped;
const char *TextAlreadyRunning;
const char *TextNoStart;
const char *TextClosing;
const char *TextOpened;
const char *TextNoOpen;
const char *TextFound;

// Error / debug messages
const char *TextPortInitError;
const char *TextInitComponent;
const char *TextPortCreated;
const char *TextPortCleaned;
const char *TextPortNotFound;

// ARexx / TimeCtrl
const char *TextSyncOn;
const char *TextSyncOff;
const char *TextSyncAlreadyOn;
const char *TextSyncAlreadyOff;
const char *TextNoResponses;
const char *TextNoClockAdjust;
const char *TextARexxCmdUnknown;
const char *TextARexxParmUnknown;
const char *TextARexxParmInvalid;
const char *TextARexxParmMiss;
const char *TextARexxVarError;
const char *TextARexxVarValue;
const char *TextARexxResult;
const char *TextARexxExecute1;
const char *TextARexxExecute2;
const char *TextARexxReceived;
const char *TextARexxShutdown;

#endif
